(ns wanijo.visualisation.viz
  (:require [dorothy
             [core :as dot]
             [jvm :as doro-jvm]]))

(defn single-instance [instance]
  (let [out-nodes (map #(vector (-> % :target :uuid)
                                {:label (-> % :target :name)})
                       (:links-out instance))
        in-nodes (map #(vector (-> % :source :uuid)
                               {:label (-> % :source :name)})
                      (:links-in instance))
        relationships-out (map (fn [{:keys [link target schema]}]
                                 [(:uuid instance) :> (:uuid target)
                                  {:label (:name link)}])
                               (:links-out instance))
        relationships-in (map (fn [{:keys [link source schema]}]
                                [(:uuid source) :> (:uuid instance)
                                 {:label (:name link)}])
                              (:links-in instance))
        rankdir (if (or (> (count in-nodes) 7)
                        (> (count out-nodes) 7))
                  "LR"
                  "TD")]
    (-> [{:rankdir rankdir}
         [(:uuid instance) {:label (:name instance)
                            :shape "cylinder"}]]
        (into out-nodes)
        (into in-nodes)
        (into relationships-out)
        (into relationships-in)
        dot/digraph
        dot/dot
        (doro-jvm/render {:format :svg}))))

(defn all-instances [instances]
  (let [labelfn #(str (:name %1)
                      "\n"
                      (:name %2))
        sources (map #(vector (-> % :source :uuid)
                              {:label (labelfn (:source %)
                                               (:schema %))})
                     instances)
        linked (filter :link instances)
        links (map #(vector (-> % :source :uuid)
                            :>
                            (-> % :target :uuid)
                            {:label (-> % :link :name)})
                   linked)
        targets (map #(vector (-> % :target :uuid)
                              {:label (labelfn (:target %)
                                               (:target_schema %))})
                     linked)]
    (-> [{:forcelabels "true"
          :rankdir "LR"}]
        (into sources)
        (into targets)
        (into links)
        dot/digraph
        dot/dot
        (doro-jvm/render {:format :svg}))))
